@@ -20,6 +20,7 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 from __future__ import print_function
 
+from distutils.version import LooseVersion
 import logging # pylint: disable=unused-import
 import re
 import os
@@ -98,7 +99,7 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
     version: The version of the PGI compiler to use.  Note this value
     is currently only used when setting the environment and does not
     control the version of the compiler downloaded.  The default value
-    is `18.10`.
+    is `19.4`.
 
     # Examples
 
@@ -122,6 +123,7 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
         super(pgi, self).__init__(**kwargs)
 
         self.__commands = [] # Filled in by __setup()
+        self.__libnuma_path = '' # Filled in __distro()
         self.__runtime_commands = [] # Filled in by __setup()
 
         # By setting this value to True, you agree to the PGI End-User
@@ -135,15 +137,17 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
         self.__prefix = kwargs.get('prefix', '/opt/pgi')
         self.__referer = r'https://www.pgroup.com/products/community.htm?utm_source=hpccm\&utm_medium=wgt\&utm_campaign=CE\&nvid=nv-int-14-39155'
         self.__system_cuda = kwargs.get('system_cuda', False)
+        self.__system_libnuma = kwargs.get('system_libnuma', True)
         self.__tarball = kwargs.get('tarball', '')
         self.__url = 'https://www.pgroup.com/support/downloader.php?file=pgi-community-linux-x64'
 
         # The version is fragile since the latest version is
         # automatically downloaded, which may not match this default.
-        self.__version = kwargs.get('version', '18.10')
+        self.__version = kwargs.get('version', '19.4')
         self.__wd = '/var/tmp' # working directory
 
         self.__basepath = os.path.join(self.__prefix, 'linux86-64')
+        self.__basepath_llvm = os.path.join(self.__prefix, 'linux86-64-llvm')
 
         self.toolchain = toolchain(CC='pgcc', CXX='pgc++', F77='pgfortran',
                                    F90='pgfortran', FC='pgfortran')
@@ -181,11 +185,21 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
         if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
             if not self.__ospackages:
                 self.__ospackages = ['gcc', 'g++', 'libnuma1', 'perl']
-                self.__runtime_ospackages = ['libnuma1']
+                if self.__mpi:
+                    self.__ospackages.append('openssh-client')
+            self.__runtime_ospackages = ['libnuma1']
+            if self.__mpi:
+                self.__runtime_ospackages.append('openssh-client')
+            self.__libnuma_path = '/usr/lib/x86_64-linux-gnu'
         elif hpccm.config.g_linux_distro == linux_distro.CENTOS:
             if not self.__ospackages:
                 self.__ospackages = ['gcc', 'gcc-c++', 'numactl-libs', 'perl']
-                self.__runtime_ospackages = ['numactl-libs']
+                if self.__mpi:
+                    self.__ospackages.append('openssh-clients')
+            self.__runtime_ospackages = ['numactl-libs']
+            if self.__mpi:
+                self.__runtime_ospackages.append('openssh-clients')
+            self.__libnuma_path = '/usr/lib64'
         else:
             raise RuntimeError('Unknown Linux distribution')
 
@@ -195,11 +209,22 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
 
         pgi_path = os.path.join(self.__basepath, self.__version)
         mpi_path = os.path.join(pgi_path, 'mpi', 'openmpi')
+        if LooseVersion(self.__version) >= LooseVersion('19.4'):
+            mpi_path = os.path.join(pgi_path, 'mpi', 'openmpi-3.1.3')
 
         if runtime:
             # Runtime environment
-            e = {'LD_LIBRARY_PATH': '{}:$LD_LIBRARY_PATH'.format(
-                os.path.join(pgi_path, 'lib'))}
+            if self.__mpi:
+                # PGI MPI component is selected
+                e['LD_LIBRARY_PATH'] = '{}:{}:$LD_LIBRARY_PATH'.format(
+                    os.path.join(mpi_path, 'lib'),
+                    os.path.join(pgi_path, 'lib'))
+                e['PATH'] = '{}:$PATH'.format(
+                    os.path.join(mpi_path, 'bin'))
+            else:
+                # PGI MPI component is not selected
+                e['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(
+                    os.path.join(pgi_path, 'lib'))
         else:
             # Development environment
             if self.__extended_environment:
@@ -232,9 +257,19 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
                         os.path.join(pgi_path, 'bin'))
             else:
                 # Basic environment only
-                e = {'PATH': '{}:$PATH'.format(os.path.join(pgi_path, 'bin')),
-                     'LD_LIBRARY_PATH': '{}:$LD_LIBRARY_PATH'.format(
-                         os.path.join(pgi_path, 'lib'))}
+                if self.__mpi:
+                    e['LD_LIBRARY_PATH'] = '{}:{}:$LD_LIBRARY_PATH'.format(
+                        os.path.join(mpi_path, 'lib'),
+                        os.path.join(pgi_path, 'lib'))
+                    e['PATH'] = '{}:{}:$PATH'.format(
+                        os.path.join(mpi_path, 'bin'),
+                        os.path.join(pgi_path, 'bin'))
+                else:
+                    # PGI MPI component is not selected
+                    e = {'PATH': '{}:$PATH'.format(os.path.join(pgi_path,
+                                                                'bin')),
+                         'LD_LIBRARY_PATH': '{}:$LD_LIBRARY_PATH'.format(
+                             os.path.join(pgi_path, 'lib'))}
 
         return e
 
@@ -301,19 +336,35 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
         self.__commands.append(r'echo "variable library_path is default(\$if(\$LIBRARY_PATH,\$foreach(ll,\$replace(\$LIBRARY_PATH,":",), -L\$ll)));" >> {}'.format(siterc))
         self.__commands.append(r'echo "append LDLIBARGS=\$library_path;" >> {}'.format(siterc))
 
+        # Override the installer behavior and force the use of the
+        # system libnuma library
+        if self.__system_libnuma:
+            self.__commands.append('ln -sf {0} {1}'.format(
+                os.path.join(self.__libnuma_path, 'libnuma.so.1'),
+                os.path.join(self.__basepath, self.__version, 'lib',
+                             'libnuma.so')))
+            self.__commands.append('ln -sf {0} {1}'.format(
+                os.path.join(self.__libnuma_path, 'libnuma.so.1'),
+                os.path.join(self.__basepath, self.__version, 'lib',
+                             'libnuma.so.1')))
+
+        # Cleanup
         self.__commands.append(self.cleanup_step(
             items=[os.path.join(self.__wd, tarball),
                    os.path.join(self.__wd, 'pgi')]))
 
-        # Runtime workaround for libnuma issue impacting version 18.4
-        # on Ubuntu
-        if (self.__version == '18.4' and
-            hpccm.config.g_linux_distro == linux_distro.UBUNTU):
-            self.__runtime_commands.append('ln -s {0} {1}'.format(
-                '/usr/lib/x86_64-linux-gnu/libnuma.so.1',
-                os.path.join(self.__basepath, self.__version, 'lib',
-                             'libnuma.so')))
-
+        # libnuma.so and libnuma.so.1 must be symlinks to the system
+        # libnuma library.  They are originally symlinks, but Docker
+        # "COPY -from" copies the file pointed to by the symlink,
+        # converting them to files, so recreate the symlinks.
+        self.__runtime_commands.append('ln -sf {0} {1}'.format(
+            os.path.join(self.__libnuma_path, 'libnuma.so.1'),
+            os.path.join(self.__basepath, self.__version, 'lib',
+                         'libnuma.so')))
+        self.__runtime_commands.append('ln -sf {0} {1}'.format(
+            os.path.join(self.__libnuma_path, 'libnuma.so.1'),
+            os.path.join(self.__basepath, self.__version, 'lib',
+                         'libnuma.so.1')))
 
     def runtime(self, _from='0'):
         """Generate the set of instructions to install the runtime specific
@@ -329,15 +380,39 @@ class pgi(bb_base, hpccm.templates.rm, hpccm.templates.tar,
         """
         instructions = []
         instructions.append(comment('PGI compiler'))
+
         if self.__runtime_ospackages:
             instructions.append(packages(ospackages=self.__runtime_ospackages))
+
+        pgi_path = os.path.join(self.__basepath, self.__version)
+        src_path = pgi_path
+        if LooseVersion(self.__version) >= LooseVersion('19.4'):
+            # Too many levels of symlinks for the Docker builder to
+            # handle, so use the real path
+            src_path = os.path.join(self.__basepath_llvm, self.__version)
         instructions.append(copy(_from=_from,
-                                 src=os.path.join(self.__basepath,
-                                                  self.__version,
-                                                  'REDIST', '*.so'),
-                                 dest=os.path.join(self.__basepath,
-                                                   self.__version,
+                                 src=os.path.join(src_path,
+                                                  'REDIST', '*.so*'),
+                                 dest=os.path.join(pgi_path,
                                                    'lib', '')))
+
+        # REDIST workaround for incorrect libcudaforwrapblas.so
+        # symlink
+        if LooseVersion(self.__version) >= LooseVersion('18.10'):
+            instructions.append(
+                copy(_from=_from,
+                     src=os.path.join(pgi_path, 'lib',
+                                      'libcudaforwrapblas.so'),
+                     dest=os.path.join(pgi_path, 'lib',
+                                       'libcudaforwrapblas.so')))
+
+        if self.__mpi:
+            mpi_path = os.path.join(pgi_path, 'mpi', 'openmpi')
+            if LooseVersion(self.__version) >= LooseVersion('19.4'):
+                mpi_path = os.path.join(pgi_path, 'mpi', 'openmpi-3.1.3')
+            instructions.append(copy(_from=_from,
+                                     src=mpi_path, dest=mpi_path))
+
         if self.__runtime_commands:
             instructions.append(shell(commands=self.__runtime_commands))
 
