@@ -24,10 +24,11 @@ import logging # pylint: disable=unused-import
 
 import hpccm.config
 
+from hpccm.building_blocks.base import bb_base
 from hpccm.common import linux_distro
 from hpccm.primitives.shell import shell
 
-class yum(object):
+class yum(bb_base):
     """The `yum` building block specifies the set of operating system
     packages to install.  This building block should only be used on
     images that use the Red Hat package manager (e.g., CentOS).
@@ -62,7 +63,7 @@ class yum(object):
     def __init__(self, **kwargs):
         """Initialize building block"""
 
-        #super(yum, self).__init__()
+        super(yum, self).__init__()
 
         self.__commands = []
         self.__epel = kwargs.get('epel', False)
@@ -78,9 +79,12 @@ class yum(object):
         # block
         self.__setup()
 
-    def __str__(self):
-        """String representation of the building block"""
-        return str(shell(chdir=False, commands=self.__commands))
+        # Fill in container instructions
+        self.__instructions()
+
+    def __instructions(self):
+        """Fill in container instructions"""
+        self += shell(chdir=False, commands=self.__commands)
 
     def __setup(self):
         """Construct the series of commands to execute"""
